/*****************************************************************************

    Name: init.c
    Description: fuzzy search library - init FDB structure
    
                 
    Author: Oleg Khovayko,  http://olegh.spedia.net

   ***************************************************************************

                          PUBLIC DOMAIN NOTICE
              National Center for Biotechnology Information

    This software/database is a "United States Government Work" under the
    terms of the United States Copyright Act.  It was written as part of    
    the author's official duties as a United States Government employee
    and thus cannot be copyrighted.  This software/database is freely
    available to the public for use. The National Library of Medicine and
    the U.S. Government have not placed any restriction on its use or
    reproduction.

    Although all reasonable efforts have been taken to ensure the accuracy
    and reliability of the software and data, the NLM and the U.S.
    Government do not and cannot warrant the performance or results that
    may be obtained by using this software or data. The NLM and the U.S.
    Government disclaim all warranties, express or implied, including
    warranties of performance, merchantability or fitness for any
    particular purpose.

    Please cite the author in any work or product based on this material.

   ***************************************************************************

    Entry Points:



*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <fuzzy.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <strings.h>
/*--------------------------------------------------------------------------*/

static void *mmap_open(const char *dir, const char *fname) {
  char full_fname[4000];
  int fd;
  struct stat buf;

  sprintf(full_fname, "%s/%s", dir, fname);

  return ((fd = open(full_fname, O_RDONLY)) < 0 || fstat(fd, &buf) < 0)?
    NULL : 
    mmap(NULL, buf.st_size, PROT_READ, MAP_SHARED | MAP_NORESERVE, fd, 0);

} /* mmap_open */

/*--------------------------------------------------------------------------*/
int fuzzy_setup(FDB_HANDLE *h, const char *db_name, const char *path) {
  bzero(h, sizeof(FDB_HANDLE));
  h->db_name = strdup(db_name);
  h->path    = strdup(path);
  return  h->db_name != NULL && h->path != NULL;
}


/*--------------------------------------------------------------------------*/

int fuzzy_init(FDB_HANDLE *h) {

  if(h == NULL)
    return 0;

  if(h->xrs_table == NULL) {
    h->xrs_table       = (Uint4*)mmap_open(h->path, "triades.xrs");
    h->statmas_entry   = (XRS_ENTRY*)mmap_open(h->path, "triades.xrs.entry");
    h->termlist        = (const char*)mmap_open(h->path, "termlist.txt");
    h->termlist_entry = (const TERM_ENTRY*)mmap_open(h->path, "termlist.txt.entry");
    if(!(h->xrs_table && h->statmas_entry 
         && h->termlist && h->termlist_entry))
      return 0;
  }

  return 1;
}

/*--------------------------------------------------------------------------*/

/*==================================================================================*/

