/* $Id: radix50.c,v 1.1 2000/08/07 18:09:42 olegh Exp $ */
/*****************************************************************************

    Description: Radix50 encoder/decoder

    Author: Oleg Khovayko

   ***************************************************************************

                          PUBLIC DOMAIN NOTICE
              National Center for Biotechnology Information

    This software/database is a "United States Government Work" under the
    terms of the United States Copyright Act.  It was written as part of    
    the author's official duties as a United States Government employee
    and thus cannot be copyrighted.  This software/database is freely
    available to the public for use. The National Library of Medicine and
    the U.S. Government have not placed any restriction on its use or
    reproduction.

    Although all reasonable efforts have been taken to ensure the accuracy
    and reliability of the software and data, the NLM and the U.S.
    Government do not and cannot warrant the performance or results that
    may be obtained by using this software or data. The NLM and the U.S.
    Government disclaim all warranties, express or implied, including
    warranties of performance, merchantability or fitness for any
    particular purpose.

    Please cite the author in any work or product based on this material.

   ***************************************************************************

    Entry Points:

    Modification History:
        02 Mar 2000 - olegh - originally written

    Bugs and restriction on use:

    Notes:

*****************************************************************************/

/****************************************************************************/
/* Includes */
#include <fuzzy.h>

/****************************************************************************/
static const Uint1 char_to_sym[] = {
    /*00, 01, 02, 03, 04, 05, 06, 07, 10, 11, 12, 13, 14, 15, 16, 17 */
     035,035,035,035,035,035,035,035,035,035,035,035,035,035,035,035, 

    /*20, 21, 22, 23, 24, 25, 26, 27, 30, 31, 32, 33, 34, 35, 36, 37 */
     035,035,035,035,035,035,035,035,035,035,035,035,035,035,035,035, 

    /*SP,  !   "   #   $   %   &   '   (   )   *   +   ,   -   .   / */
     000,035,035,035,033,035,035,035,035,035,035,035,035,035,034,035,

    /* 0   1   2   3   4   5   6   7   8   9   :   ;   <   =   >   ? */
     036,037,040,041,042,043,044,045,047,047,035,035,035,035,035,035,

    /* @   A   B   C   D   E   F   G   H   I   J   K   L   M   N   O */
     035,001,002,003,004,005,006,007,010,011,012,013,014,015,016,017,

    /* P   Q   R   S   T   U   V   W   X   Y   Z   [   \   ]   ^   _*/
     020,021,022,023,024,025,026,027,030,031,032,035,035,035,035,035,

    /* `   a   b   c   d   e   f   g   h   i   j   k   l   m   n   o */
     035,001,002,003,004,005,006,007,010,011,012,013,014,015,016,017,

    /* p   q   r   s   t   u   v   w   x   y   z   {   |   }   ~ DEL */
     020,021,022,023,024,025,026,027,030,031,032,035,035,035,035,035
};

static const char sym_to_char[] = 
  " abcdefghijklmnopqrstuvwxyz$.-0123456789";

R50 triade_to_r50(const char *str) {
    static const Uint2 mul_k[] = { 0, 050, 050*050, 0 };
    Uint2 rc = 0;
    register char c;
    int cnt = 3;

    do {
        if((c = *str++) == EOLN) {
            return rc * mul_k[cnt];
        } /* if */
        rc = rc * 050 + (c < 0 ? 035 : char_to_sym[c]);
    } while(--cnt);

    return rc;
} /* triade_to_r50 */


void r50_to_triade(R50 val, char *p) {
    register char *p1 = p + 3;

    *p1 = 0;

    do {
        *--p1 = sym_to_char[val % 050];
        val /= 050;
    } while(p != p1);
} /* r50_to_triade */


/* returns outbuf len in Uint2 units */
Uint2 string_to_r50(char *str, Uint2 *outbuf) {
    return 0;
}
