/*****************************************************************************

    Name: triade.c
    Description: fuzzy search library (triade processor)
    
                 
    Author: Oleg Khovayko,  http://olegh.spedia.net

   ***************************************************************************

                          PUBLIC DOMAIN NOTICE
              National Center for Biotechnology Information

    This software/database is a "United States Government Work" under the
    terms of the United States Copyright Act.  It was written as part of    
    the author's official duties as a United States Government employee
    and thus cannot be copyrighted.  This software/database is freely
    available to the public for use. The National Library of Medicine and
    the U.S. Government have not placed any restriction on its use or
    reproduction.

    Although all reasonable efforts have been taken to ensure the accuracy
    and reliability of the software and data, the NLM and the U.S.
    Government do not and cannot warrant the performance or results that
    may be obtained by using this software or data. The NLM and the U.S.
    Government disclaim all warranties, express or implied, including
    warranties of performance, merchantability or fitness for any
    particular purpose.

    Please cite the author in any work or product based on this material.

   ***************************************************************************

    Entry Points:



*****************************************************************************/

/****************************************************************************/
/* Includes */
#include <fuzzy.h>

#include <stdlib.h>
#include <string.h>

/****************************************************************************/
void cpy_to_canonic(const char *instr, char *outstr) {
    char c;
    int need_space = 0;
    int left_bytes = MAXTERMLEN - 1;

    while((c = *instr) <= SP && c != 0 && left_bytes > 0) instr++;

    while(c = *instr++) {
        if(c <= SP) {
            need_space = 1;
            continue;
        }

        if(need_space) {
            need_space = 0;
            *outstr++ = SP;
            left_bytes--;
        }

        *outstr++ = c;
        left_bytes--;

    } /* while */

    *outstr = EOLN;

} /* cpy_to_canonic */


Uint2 calculate_triades(const char *str, TRIADE_QTY *qty_buf, int mul_prefix) {

    static Uint1   *cnt_buf = NULL;
           R50     tmpbuf[5 * MAXTERMLEN];

     const char   *p_end     = str + strlen(str);
     const char   *p         = str;
           char    triade[3];
           int     tmpbufN   = 0;
           int     qty_bufN  = 0;


    if(cnt_buf == NULL && (cnt_buf = (Uint1*)calloc(MAXR50, 1)) == NULL)
        return 0;

    if((p_end - p) > 2)
      p_end -= 1;

    while(p < p_end) {
      cnt_buf[tmpbuf[tmpbufN++] = triade_to_r50(p++)]++;
    }

    p = str - 1;
    do {
        p++;

        triade[0] = p[0];
        triade[1] = p[1]; 
        triade[2] = '$'; 
        cnt_buf[tmpbuf[tmpbufN++] = triade_to_r50(triade)] += mul_prefix;

        triade[1] = '$'; 
        cnt_buf[tmpbuf[tmpbufN++] = triade_to_r50(triade)] += 1;

    } while(p = strchr(p, SP));

    while(--tmpbufN >= 0) {
        Uint2 triade, qty;
        triade = tmpbuf[tmpbufN];
        qty    = cnt_buf[triade];
        if(qty) {
           qty_buf[qty_bufN].triade_r50 = triade;
           qty_buf[qty_bufN].qty        = qty;
           cnt_buf[triade] = 0; 
           qty_bufN++;
        }
    } /* while */

    return qty_bufN;

} /* calculate_triades */

Uint2 calculate_noncanonic_triades(const char *str, TRIADE_QTY *qty_buf, int mul_prefix) {
    char buf[MAXTERMLEN];
    cpy_to_canonic(str, buf);
    return calculate_triades(buf, qty_buf, mul_prefix);
    
} /* calculate_noncanonic_triades */

